/*
 * Copyright 2003 Red Hat, Inc.
 * Copyright 2007-2016 Ralf Habacker
 * Copyright 2014-2018 Collabora Ltd.
 * Copyright 2016 Yiyang Fei
 *
 * Licensed under the Academic Free License version 2.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef TEST_DISABLE_CRASH_HANDLING_H
#define TEST_DISABLE_CRASH_HANDLING_H

void _dbus_disable_crash_handling (void);

#endif
